@@ -17,9 +17,11 @@ import io
 import logging
 import os
 import re
+import sys
 import tempfile
 import time
 from collections import namedtuple
+from contextlib import contextmanager
 from datetime import datetime
 from functools import lru_cache
 from operator import attrgetter
@@ -29,6 +31,8 @@ from packaging import version
 
 from ibm_watson_machine_learning import APIClient
 
+LOGNAME = 'log.txt'
+
 
 class Error(Exception):
     """Base class for all errors in this script"""
@@ -82,8 +86,6 @@ def _new_params():
     result = _the_old_params()
     # Add the filter, if one is required
     if _the_filter:
-        # The filter doesn't work correctly if it contains spaces
-        assert(_the_filter.find(' ') == -1)
         result['include'] = _the_filter
     return result
 
@@ -189,6 +191,18 @@ pp being integers."""
     return version.parse(current) >= version.parse(minimum)
 
 
+@contextmanager
+def suppress_stdout():
+    """"Sometimes it's nicer to not get printed output from APIClient"""
+    with open(os.devnull, "w") as devnull:
+        old_stdout = sys.stdout
+        sys.stdout = devnull
+        try:
+            yield
+        finally:
+            sys.stdout = old_stdout
+
+
 class DOWMLLib:
     """A Python client to run DO models on WML"""
 
@@ -313,28 +327,100 @@ class DOWMLLib:
         self._logger.info(f'Job id: {job_id}')
         return job_id
 
+    def client_data_asset_download(self, asset_id, filename):
+        self._logger.debug(f'Downloading asset {asset_id} in {filename}...')
+        with suppress_stdout():
+            # The return value is useless when filename is an absolute path
+            _ = self._client.data_assets.download(asset_id, filename)
+        self._logger.debug(f'Done.')
+
     def get_log(self, job_id):
         """Extract the engine log from the job.
 
         :param job_id: The id of the job to get the log from
         :return: The decoded log, or None
         """
+
+        def _get_asset_content(asset_id):
+            with tempfile.TemporaryDirectory() as temp_dir_name:
+                filename = os.path.join(temp_dir_name, f'{asset_id}-log.txt')
+                self.client_data_asset_download(asset_id, filename)
+                with open(filename) as f:
+                    content = f.read()
+                    return content
+
+        def _get_log_from_output_references(references):
+            self._logger.debug(f'Looking for {LOGNAME} in output_data_references...')
+            for ref in references:
+                if ref.get('type') != 'data_asset':
+                    continue
+                if 'id' not in ref:
+                    self._logger.warning(f'Ignoring data asset with no id')
+                    continue
+                if ref['id'] == LOGNAME:
+                    self._logger.debug(f'Found it.')
+                    try:
+                        asset_id = ref['location']['id']
+                    except KeyError:
+                        self._logger.error('Log data asset has no location/id information')
+                        break
+                    self._logger.debug(f'This is asset {asset_id}.')
+                    return _get_asset_content(asset_id)
+            return None
+
+        def _get_log_from_outputs(outputs):
+            self._logger.debug(f'Looking for {LOGNAME} in output_data...')
+            for output_data in outputs:
+                if output_data['id'] == LOGNAME:
+                    if 'content' not in output_data:
+                        self._logger.error(f'Log without content for job {job_id}')
+                        continue
+                    self._logger.debug('Found it. Decoding it...')
+                    output = output_data['content']
+                    output = self.decode_log(output)
+                    output = self.remove_empty_lines(output)
+                    self._logger.debug('Decoded the log.')
+                    return output
+            return None
+
         job_details = self.get_job_details(job_id, with_contents='log')
         try:
-            outputs = job_details['entity']['decision_optimization']['output_data']
+            do = job_details['entity']['decision_optimization']
         except KeyError:
-            self._logger.warning('No output structure available for this job')
+            self._logger.warning('No decision_optimization structure available for this job')
             return None
-        for output_data in outputs:
-            if output_data['id'] == 'log.txt':
-                if 'content' not in output_data:
-                    self._logger.error(f'Log without content for job {job_id}')
-                    continue
-                output = output_data['content']
-                output = self.decode_log(output)
-                output = self.remove_empty_lines(output)
-                return output
-        return None
+        # When we have references in the job, the 'output_data' may be an empty list
+        if 'output_data' in do and do['output_data']:
+            return _get_log_from_outputs(do['output_data'])
+        elif 'output_data_references' in do:
+            return _get_log_from_output_references(do['output_data_references'])
+        else:
+            self._logger.warning('No output_data or output_data_references structure available for this job')
+            return None
+
+    def get_output_assets(self, details):
+        """"Extract the output data asset ids from the job.
+
+        :param details: The details of the job to get the output from
+        :return: A list of outputs. Each output is a tuple (name, id)
+        where the name is, well, the name of the output, and id is the
+        id of the corresponding data asset.
+        """
+        result = {}
+        try:
+            refs = details['entity']['decision_optimization']['output_data_references']
+        except KeyError:
+            self._logger.debug('No output references structure available for this job')
+            return result
+        for ref in refs:
+            type = ref.get('type')
+            if type != 'data_asset':
+                self._logger.debug(f'Ignoring asset of unkown type \'{type}\'.')
+                continue
+            name = ref['id']
+            self._logger.debug(f'Found a data asset named {name}.')
+            result[name] = ref['location']['id']
+        return result
 
     def get_output(self, details, csv_as_dataframe=None, tabular_as_csv=False):
         """"Extract the outputs from the job.
@@ -345,10 +431,9 @@ class DOWMLLib:
         instead
         :param tabular_as_csv: Whether tabular outputs should be returned as
         CSV file content instead of Pandas dataframe
-        :return: A list of outputs. Each output is a tuple (name, content)
-        where the name is, well, the name of the output, and content is the
-        decoded content, as bytes. We don't assume that the content is actually
-        text.
+        :return: A dict of outputs, with the names of the assets as keys, and
+        the content as value, as bytes. We don't assume that the content is
+        actually text.
         """
         if csv_as_dataframe is not None:
             # We have a non-default value for this deprecated parameter
@@ -419,7 +504,7 @@ class DOWMLLib:
         if not with_contents:
             output_filter = 'solve_parameters,solve_state,status'
         elif with_contents == 'log':
-            output_filter = 'output_data'
+            output_filter = 'output_data,output_data_references'
         job_details = self.client_get_job_details(client, job_id, output_filter)
         self._logger.debug('Done.')
         if with_contents != 'full' and with_contents != 'log':
@@ -428,19 +513,24 @@ class DOWMLLib:
 
     @staticmethod
     def client_get_job_details(client, job_id, with_filter=None):
-        global _the_filter
-        global _the_old_params
-        # Save the filter in a global variable for our new function to find it
-        _the_filter = with_filter
-        # Save the pointer to the original code
-        _the_old_params = client._params
-        # and replace it with our new function
-        client._params = _new_params
-        try:
-            result = client.deployments.get_job_details(job_id)
-        finally:
-            # Put back the original code
-            client._params = _the_old_params
+        # The filter doesn't work correctly if it contains spaces
+        assert(not with_filter or (with_filter.find(' ') == -1))
+        if version_is_greater(client.version, "1.0.154"):
+            result = client.deployments.get_job_details(job_id, with_filter)
+        else:
+            global _the_filter
+            global _the_old_params
+            # Save the filter in a global variable for our new function to find it
+            _the_filter = with_filter
+            # Save the pointer to the original code
+            _the_old_params = client._params
+            # and replace it with our new function
+            client._params = _new_params
+            try:
+                result = client.deployments.get_job_details(job_id)
+            finally:
+                # Put back the original code
+                client._params = _the_old_params
         return result
 
     @staticmethod
